@@ -30,7 +30,7 @@ namespace KDevelop {
   class QualifiedIdentifier;
   class DUContext;
   class TopDUContext;
-  class HashedString;
+  class IndexedString;
   class SimpleRange;
 }
 
@@ -95,8 +95,8 @@ public:
   
   ClassDeclaration(const ClassDeclaration& rhs);
   ClassDeclaration(ClassDeclarationData& data);
-  ClassDeclaration(const KDevelop::SimpleRange& range, KDevelop::DUContext* context);
-  ClassDeclaration(ClassDeclarationData& data, const KDevelop::SimpleRange& range, KDevelop::DUContext* context);
+  ClassDeclaration(const KDevelop::RangeInRevision& range, KDevelop::DUContext* context);
+  ClassDeclaration(ClassDeclarationData& data, const KDevelop::RangeInRevision& range, KDevelop::DUContext* context);
   ~ClassDeclaration();
 
   void clearBaseClasses();
