@@ -26,7 +26,6 @@
 #include <QtCore/QVector>
 #include <util/kdevvarlengtharray.h>
 
-#include "../editor/documentcursorobject.h"
 #include "identifier.h"
 #include "duchainbase.h"
 #include "types/abstracttype.h"
@@ -36,13 +35,6 @@
 
 class QWidget;
 
-///@todo Move the complete handling of smart-ranges out of here, so that the duchain is independent of the editor.
-///           Use the foreground-lock instead of the smart-lock everywhere.
-
-namespace KTextEditor {
-  class SmartRange;
-}
-
 namespace KDevelop
 {
 
@@ -53,6 +45,13 @@ class TopDUContext;
 class DUContext;
 class DUContextData;
 
+class KDEVPLATFORMLANGUAGE_EXPORT DUChainVisitor {
+public:
+  virtual void visit(DUContext* context) = 0;
+  virtual void visit(Declaration* declaration) = 0;
+  virtual ~DUChainVisitor();
+};
+
 ///Represents a context only by its global indices
 class KDEVPLATFORMLANGUAGE_EXPORT IndexedDUContext {
   public:
@@ -179,14 +178,14 @@ class ImportTraceItem
 {
 public:
 
-  ImportTraceItem(const DUContext* _ctx, SimpleCursor _pos = SimpleCursor::invalid()) : ctx(_ctx), position(_pos) {
+  ImportTraceItem(const DUContext* _ctx, CursorInRevision _pos = CursorInRevision::invalid()) : ctx(_ctx), position(_pos) {
   }
   ImportTraceItem() {
   }
 
   //The trace goes backwards. This means that for each imported context, it contains the context the new one is imported to, not the imported context.
   const DUContext* ctx;
-  SimpleCursor position;
+  CursorInRevision position;
 };
 
 class ImportTrace : public KDevVarLengthArray<ImportTraceItem, 40>
@@ -228,7 +227,7 @@ public:
    *
    * If the parent is in the symbol table and the context is not anonymous, it will also be added to the symbol table. You nead a write-lock to the DUChain then
    */
-  explicit DUContext(const SimpleRange& range, DUContext* parent = 0, bool anonymous = false);
+  explicit DUContext(const RangeInRevision& range, DUContext* parent = 0, bool anonymous = false);
   explicit DUContext(DUContextData&);
 
   /**
@@ -292,23 +291,38 @@ public:
    */
   virtual TopDUContext* topContext() const;
 
+  /**
+   * Visits all duchain objects in the whole duchain. Classes that hold a unique link to duchain objects
+   * like instantiations have to pass the visitor over to those classes.
+   * */
+  virtual void visit(DUChainVisitor& visitor);
+  
   /**
    * Find the context which most specifically covers \a position.
    * @param includeRightBorder When this is true, contexts will also be found that have the position on their right border.
    * The search is recursive, so the most specific context is found.
+   * 
+   * @warning This uses the ranges in the local revision of the document (at last parsing time).
+   *                  Use DUChainBase::transformToLocalRevision to transform the cursor into that revision first.
    */
-  DUContext* findContextAt(const SimpleCursor& position, bool includeBorders = false) const;
+  DUContext* findContextAt(const CursorInRevision& position, bool includeBorders = false) const;
 
   /**
    * Find a child declaration that has a rang that covers the given position
    * The search is local, not recursive.
+   * 
+   * @warning This uses the ranges in the local revision of the document (at last parsing time).
+   *                  Use DUChainBase::transformToLocalRevision to transform the cursor into that revision first.
    */
-  Declaration* findDeclarationAt(const SimpleCursor& position) const;
+  Declaration* findDeclarationAt(const CursorInRevision& position) const;
 
   /**
    * Find the context which most specifically covers \a range.
+   * 
+   * @warning This uses the ranges in the local revision of the document (at last parsing time).
+   *                  Use DUChainBase::transformToLocalRevision to transform the cursor into that revision first.
    */
-  DUContext* findContextIncluding(const SimpleRange& range) const;
+  DUContext* findContextIncluding(const RangeInRevision& range) const;
 
   /**
    * Calculate the fully qualified scope identifier
@@ -355,10 +369,10 @@ public:
   /// Represents an imported parent context.
   struct KDEVPLATFORMLANGUAGE_EXPORT Import {
     ///DUChain must be read-locked when this is called
-    Import(DUContext* context, const DUContext* importer, const SimpleCursor& position = SimpleCursor::invalid());
-    Import() : position(SimpleCursor::invalid()) {
+    Import(DUContext* context, const DUContext* importer, const CursorInRevision& position = CursorInRevision::invalid());
+    Import() : position(CursorInRevision::invalid()) {
     }
-    Import(const DeclarationId& id, const SimpleCursor& position = SimpleCursor::invalid());
+    Import(const DeclarationId& id, const CursorInRevision& position = CursorInRevision::invalid());
     bool operator==(const Import& rhs) const {
       return m_context == rhs.m_context && m_declaration == rhs.m_declaration;
     }
@@ -384,7 +398,7 @@ public:
       return m_declaration;
     }
 
-    SimpleCursor position;
+CursorInRevision position;
     private:
       //Either we store m_declaration, or m_context. That way we can resolve specialized contexts.
       ///@todo Compress using union
@@ -405,12 +419,12 @@ public:
    * addImportedParentContext(..) was called with a valid cursor, this will return that position.
    * Else an invalid cursor is returned.
    * */
-  virtual SimpleCursor importPosition(const DUContext* target) const;
+  virtual CursorInRevision importPosition(const DUContext* target) const;
 
   /**
    * Returns true if this context imports @param origin at any depth, else false.
    * */
-  virtual bool imports(const DUContext* origin, const SimpleCursor& position = SimpleCursor::invalid()) const;
+  virtual bool imports(const DUContext* origin, const CursorInRevision& position = CursorInRevision::invalid()) const;
 
   /**
    * Adds an imported context.
@@ -423,7 +437,7 @@ public:
    * \note Be sure to have set the text location first, so that
    * the chain is sorted correctly.
    */
-  virtual void addImportedParentContext(DUContext* context, const SimpleCursor& position = SimpleCursor::invalid(), bool anonymous = false, bool temporary = false);
+  virtual void addImportedParentContext(DUContext* context, const CursorInRevision& position = CursorInRevision::invalid(), bool anonymous = false, bool temporary = false);
 
   /**
    * Adds an imported context, which may be indirect.
@@ -503,7 +517,7 @@ public:
    *
    * \warning this may return declarations which are not in this tree, you may need to lock them too...
    */
-  QList<Declaration*> findDeclarations(const QualifiedIdentifier& identifier, const SimpleCursor& position = SimpleCursor::invalid(), const AbstractType::Ptr& dataType = AbstractType::Ptr(), const TopDUContext* topContext = 0, SearchFlags flags = NoSearchFlags) const;
+  QList<Declaration*> findDeclarations(const QualifiedIdentifier& identifier, const CursorInRevision& position = CursorInRevision::invalid(), const AbstractType::Ptr& dataType = AbstractType::Ptr(), const TopDUContext* topContext = 0, SearchFlags flags = NoSearchFlags) const;
 
   /**
    * Searches for and returns a declaration with a given \a identifier in this context, which
@@ -519,7 +533,7 @@ public:
    *
    * \overload
    */
-  QList<Declaration*> findDeclarations(const Identifier& identifier, const SimpleCursor& position = SimpleCursor::invalid(), const TopDUContext* topContext = 0, SearchFlags flags = NoSearchFlags) const;
+  QList<Declaration*> findDeclarations(const Identifier& identifier, const CursorInRevision& position = CursorInRevision::invalid(), const TopDUContext* topContext = 0, SearchFlags flags = NoSearchFlags) const;
 
   /**
    * Returns the type of any \a identifier defined in this context, or
@@ -527,7 +541,7 @@ public:
    *
    * Does not search imported parent-contexts(like base-classes).
    */
-  QList<Declaration*> findLocalDeclarations(const Identifier& identifier, const SimpleCursor& position = SimpleCursor::invalid(), const TopDUContext* topContext = 0, const AbstractType::Ptr& dataType = AbstractType::Ptr(), SearchFlags flags = NoSearchFlags) const;
+  QList<Declaration*> findLocalDeclarations(const Identifier& identifier, const CursorInRevision& position = CursorInRevision::invalid(), const TopDUContext* topContext = 0, const AbstractType::Ptr& dataType = AbstractType::Ptr(), SearchFlags flags = NoSearchFlags) const;
 
   /**
    * Clears all local declarations. Does not delete the declaration; the caller
@@ -557,7 +571,7 @@ public:
    *
    * \returns the requested context if one was found, otherwise null.
    */
-  DUContext* findContext(const SimpleCursor& position, DUContext* parent = 0) const;
+  DUContext* findContext(const CursorInRevision& position, DUContext* parent = 0) const;
 
   /**
    * Iterates the tree to see if the provided \a context is a subcontext of this context.
@@ -580,7 +594,7 @@ public:
    *
    * \returns the requested declarations, if any were active at that location. Declarations propagated into this context(@see setPropagateDeclarations) are included.
    */
-  QList< QPair<Declaration*, int> > allDeclarations(const SimpleCursor& position, const TopDUContext* topContext, bool searchInParents=true) const;
+  QList< QPair<Declaration*, int> > allDeclarations(const CursorInRevision& position, const TopDUContext* topContext, bool searchInParents=true) const;
 
   /**
    * Delete and remove all slaves(uses, declarations, definitions, contexts) that are not in the given set
@@ -602,16 +616,10 @@ public:
     *
     * The actual uses are stored within DUContext, where each use consists of a range and the declaration-index of
     * the used declaration.
-    *
-    * To save memory, smart-ranges are only attached to the uses when the document is actually loaded.
-    * Uses need to be ordered by their appearance.
     * */
 
   /**
    * Return a vector of all uses which occur in this context.
-   * When the uses have smart-ranges attached, those are synced in the moment that uses() is called, so you should
-   * call this immediately before using the returned ranges, and you should hold the smart-lock while calling this
-   * if you want to compare the ranges, so you can be sure they aren't changed in the meantime.
    * To get the actual declarations, use TopDUContext::usedDeclarationForIndex(..) with the declarationIndex.
    */
   const Use* uses() const;
@@ -623,24 +631,13 @@ public:
    * Find the use which encompasses \a position, if one exists.
    * @return The local index of the use, or -1
    */
-  int findUseAt(const SimpleCursor& position) const;
+  int findUseAt(const CursorInRevision& position) const;
 
+  
   /**
-   * Returns the SmartRange assigned to the given use, or zero.
+   * The change must not break the ordering
    * */
-  KTextEditor::SmartRange* useSmartRange(int useIndex);
-
-  /**
-   * Assigns the given SmartRange to the given use.
-   * If one use gets a smart range, all uses need to get a smart range.
-   * The ownership of the range is given to this context.
-   * */
-  void setUseSmartRange(int useIndex, KTextEditor::SmartRange* range);
-
-  /**
-   * Clears all smart ranges associated with uses.
-   */
-  void clearUseSmartRanges();
+  void changeUseRange(int useIndex, const RangeInRevision& range);
 
   /**
    * Assigns the declaration represented by @param declarationIndex to the use with index @param useIndex
@@ -651,14 +648,13 @@ public:
    * Creates a new use of the declaration given  through @param declarationIndex.
    * The index must be retrieved through TopDUContext::indexForUsedDeclaration(..).
    * @param range The range of the use
-   * @param smartRange The smart range, or zero if the document is not opened.
    * @param insertBefore A hint where in the vector of uses to insert the use.
    *                     Must be correct so the order is preserved(ordered by position),
    *                     or -1 to automatically choose the position.
    *
    * @return Local index of the created use
    * */
-  int createUse(int declarationIndex, const SimpleRange& range, KTextEditor::SmartRange* smartRange, int insertBefore = -1);
+  int createUse(int declarationIndex, const RangeInRevision& range, int insertBefore = -1);
 
   /**
    * Deletes the use number @param index . index is the position in the vector of uses, not a used declaration index.
@@ -666,7 +662,7 @@ public:
   void deleteUse(int index);
 
   /**
-   * Clear and delete all uses in this context, including smart-ranges etc.
+   * Clear and delete all uses in this context.
    */
   virtual void deleteUses();
 
@@ -675,16 +671,6 @@ public:
    */
   virtual void deleteUsesRecursively();
   
-  /**
-   * Returns the smart-ranges associated to the uses
-   */
-  QVector<KTextEditor::SmartRange*> useRanges();
-  
-  /**
-   * Takes the smart-ranges away from the uses, without deleting them.
-   */
-  QVector<KTextEditor::SmartRange*> takeUseRanges();
-  
   /**
    * Can be specialized by languages to create a navigation/information-widget.
    * Ideally, the widget would be based on KDevelop::QuickOpenEmbeddedWidgetInterface
@@ -765,7 +751,7 @@ struct KDEVPLATFORMLANGUAGE_EXPORT SearchItem : public KShared {
    * */
   typedef KDevVarLengthArray<Declaration*, 40> DeclarationList;
 
-  virtual bool findDeclarationsInternal(const SearchItem::PtrList& identifiers, const SimpleCursor& position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* source, SearchFlags flags, uint depth ) const;
+  virtual bool findDeclarationsInternal(const SearchItem::PtrList& identifiers, const CursorInRevision& position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* source, SearchFlags flags, uint depth ) const;
 
   ///Call this after parsing is finished. It will optimize the internal vectors to reduce memory-usage.
   void squeeze();
@@ -786,18 +772,18 @@ struct KDEVPLATFORMLANGUAGE_EXPORT SearchItem : public KShared {
    * This includes declarations propagated from sub-contexts.
    * @param hadUrls is used to count together all contexts that already were visited, so they are not visited again.
    */
-  virtual void mergeDeclarationsInternal(QList< QPair<Declaration*, int> >& definitions, const SimpleCursor& position, QHash<const DUContext*, bool>& hadContexts, const TopDUContext* source, bool searchInParents = true, int currentDepth = 0) const;
+  virtual void mergeDeclarationsInternal(QList< QPair<Declaration*, int> >& definitions, const CursorInRevision& position, QHash<const DUContext*, bool>& hadContexts, const TopDUContext* source, bool searchInParents = true, int currentDepth = 0) const;
 
   /// Logic for calculating the fully qualified scope name
   QualifiedIdentifier scopeIdentifierInternal(DUContext* context) const;
 
-  virtual void findLocalDeclarationsInternal( const Identifier& identifier, const SimpleCursor & position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* source, SearchFlags flags ) const;
+  virtual void findLocalDeclarationsInternal( const Identifier& identifier, const CursorInRevision & position, const AbstractType::Ptr& dataType, DeclarationList& ret, const TopDUContext* source, SearchFlags flags ) const;
 
   /**Applies namespace-imports and namespace-aliases and returns possible absolute identifiers that need to be searched.
    * @param targetIdentifiers will be filled with all identifiers that should be searched for, instead of identifier.
    * @param onlyImports if this is true, namespace-aliases will not be respected, but only imports. This is faster.
    * */
-  void applyAliases(const SearchItem::PtrList& identifiers, SearchItem::PtrList& targetIdentifiers, const SimpleCursor& position, bool canBeNamespace, bool onlyImports = false) const;
+  void applyAliases(const SearchItem::PtrList& identifiers, SearchItem::PtrList& targetIdentifiers, const CursorInRevision& position, bool canBeNamespace, bool onlyImports = false) const;
   /**
    * Applies the aliases that need to be applied when moving the search from this context up to the parent-context.
    * The default-implementation adds a set of identifiers with the own local identifier prefixed, if this is a namespace.
@@ -805,7 +791,7 @@ struct KDEVPLATFORMLANGUAGE_EXPORT SearchItem : public KShared {
    * */
   virtual void applyUpwardsAliases(SearchItem::PtrList& identifiers, const TopDUContext* source) const;
 
-  DUContext(DUContextData& dd, const SimpleRange& range, DUContext* parent = 0, bool anonymous = false);
+  DUContext(DUContextData& dd, const RangeInRevision& range, DUContext* parent = 0, bool anonymous = false);
 
   ///Just uses the data from the given context(doesn't copy or change anything, and the data will not be deleted on this contexts destruction)
   DUContext(DUContext& useDataFrom);
@@ -828,15 +814,9 @@ private:
   friend class LocalIndexedDUContext;
   friend class TopDUContextDynamicData;
   
-  void synchronizeUsesFromSmart() const;
-  void synchronizeUsesToSmart() const;
-  
   void clearDeclarationIndices();
   void updateDeclarationIndices();
 
-  virtual void rangePositionChanged(KTextEditor::SmartRange* range);
-  virtual void rangeDeleted(KTextEditor::SmartRange* range);
-
   DUCHAIN_DECLARE_DATA(DUContext)
   class DUContextDynamicData* m_dynamicData;
 };
@@ -860,12 +840,7 @@ KDEVPLATFORMLANGUAGE_EXPORT Identifier& globalAliasIdentifier();
 /**
   * Collects all uses of the given @param declarationIndex
   * */
-KDEVPLATFORMLANGUAGE_EXPORT QList<SimpleRange> allUses(DUContext* context, int declarationIndex, bool noEmptyRanges = false);
-
-/**
-  * Collects the smart-ranges of all uses of the given @param declarationIndex
-  * */
-KDEVPLATFORMLANGUAGE_EXPORT QList<KTextEditor::SmartRange*> allSmartUses(DUContext* context, int declarationIndex);
+KDEVPLATFORMLANGUAGE_EXPORT QList<RangeInRevision> allUses(DUContext* context, int declarationIndex, bool noEmptyRanges = false);
 }
 
 #endif // DUCONTEXT_H
