@@ -1240,6 +1240,11 @@ IndexedQualifiedIdentifier::operator QualifiedIdentifier() const {
   return QualifiedIdentifier(index);
 }
 
+void initIdentifierRepository() {
+  identifierRepository();
+  qualifiedidentifierRepository();
+}
+
 }
 
 // kate: space-indent on; indent-width 2; tab-width 4; replace-tabs on; auto-insert-doxygen on
