@@ -1,7 +1,7 @@
 /*
  * This file is part of KDevelop
  *
- * Copyright 2007-2008 David Nolden <david.nolden.kdevelop@art-master.de>
+ * Copyright 2007-2010 David Nolden <david.nolden.kdevelop@art-master.de>
  * Copyright 2006 Hamish Rodda <rodda@kde.org>
  * Copyright 2009 Milian Wolff <mail@milianw.de>
  *
@@ -27,20 +27,21 @@
 #include <QtCore/QObject>
 #include <QtCore/QHash>
 #include <QtCore/QModelIndex>
-#include <ktexteditor/attribute.h>
-#include "../duchain/ducontext.h"
-#include "../interfaces/icodehighlighting.h"
 
-namespace KTextEditor { class SmartRange; }
+#include <ktexteditor/attribute.h>
+#include <ktexteditor/movingrange.h>
+
+#include <language/duchain/indexedstring.h>
+#include <language/duchain/ducontext.h>
+#include <language/editor/simplerange.h>
+#include <language/interfaces/icodehighlighting.h>
+#include <language/backgroundparser/documentchangetracker.h>
+
 namespace KDevelop
 {
 class DUContext;
 class Declaration;
 
-///@todo make the colorized highlighting work while smart-conversion
-
-///The kate smart-mutex must be locked when this class is used. @todo Lock the smart-mutex selectively within CodeHighlighting, to reduce GUI-Lockups
-
 typedef QVector<KDevelop::Declaration*> ColorMap;
 
 class CodeHighlighting;
@@ -86,27 +87,43 @@ enum Contexts {
 };
 };
 
+struct HighlightedRange
+{
+  RangeInRevision range;
+  KTextEditor::Attribute::Ptr attribute;
+  bool operator<(const HighlightedRange& rhs) const {
+    return range.start < rhs.range.start;
+  }
+};
+
+/**
+ * Code highlighting instance that is used to apply code highlighting to one specific top context
+ * */
+
 class KDEVPLATFORMLANGUAGE_EXPORT CodeHighlightingInstance : public HighlightingEnumContainer {
   public:
     CodeHighlightingInstance(const CodeHighlighting* highlighting) : m_useClassCache(false), m_highlighting(highlighting) {
     }
-    virtual ~CodeHighlightingInstance(){
+    virtual ~CodeHighlightingInstance() {
     }
 
-    virtual void highlightDeclaration(KDevelop::Declaration* declaration, const QColor &color) const;
-    virtual void highlightUse(KDevelop::DUContext* context, int index, const QColor &color) const;
+    virtual void highlightDeclaration(KDevelop::Declaration* declaration, const QColor &color);
+    virtual void highlightUse(KDevelop::DUContext* context, int index, const QColor &color);
+    virtual void highlightUses(KDevelop::DUContext* context);
+
+    void highlightDUChain(KDevelop::TopDUContext* context);
+    void highlightDUChain(KDevelop::DUContext* context, QHash<KDevelop::Declaration*, uint> colorsForDeclarations, ColorMap);
+
+    KDevelop::Declaration* localClassFromCodeContext(KDevelop::DUContext* context) const;
     /**
      * @param context Should be the context from where the declaration is used, if a use is highlighted.
      * */
     virtual Types typeForDeclaration(KDevelop::Declaration* dec, KDevelop::DUContext* context) const;
-
-    void highlightDUChainSimple(KDevelop::DUContext* context) const;
-    void highlightDUChain(KDevelop::DUContext* context) const;
-    void highlightDUChain(KDevelop::DUContext* context, QHash<KDevelop::Declaration*, uint> colorsForDeclarations, ColorMap) const;
-    void highlightUses(KDevelop::DUContext* context) const;
-    void outputRange( KTextEditor::SmartRange * range ) const;
-
-    KDevelop::Declaration* localClassFromCodeContext(KDevelop::DUContext* context) const;
+    /**
+     * Decides whether to apply auto-generated rainbow colors to @p dec.
+     * Default implementation only applies that to local variables in functions.
+     */
+    virtual bool useRainbowColor(KDevelop::Declaration* dec) const;
 
     //A temporary hash for speedup
     mutable QHash<KDevelop::DUContext*, KDevelop::Declaration*> m_contextClasses;
@@ -115,11 +132,15 @@ class KDEVPLATFORMLANGUAGE_EXPORT CodeHighlightingInstance : public Highlighting
     mutable QMap<KDevelop::IndexedDUContext, QHash<KDevelop::Declaration*, uint> > m_functionColorsForDeclarations;
     mutable QMap<KDevelop::IndexedDUContext, ColorMap> m_functionDeclarationsForColors;
 
-
     mutable bool m_useClassCache;
     const CodeHighlighting* m_highlighting;
+
+    QVector<HighlightedRange> m_highlight;
 };
 
+/**
+ * General class representing the code highlighting for one language
+ * */
 class KDEVPLATFORMLANGUAGE_EXPORT CodeHighlighting : public QObject, public KDevelop::ICodeHighlighting, public HighlightingEnumContainer
 {
   Q_OBJECT
@@ -130,22 +151,41 @@ class KDEVPLATFORMLANGUAGE_EXPORT CodeHighlighting : public QObject, public KDev
     CodeHighlighting(QObject* parent);
     virtual ~CodeHighlighting();
 
-    void highlightDUChain(KDevelop::TopDUContext* context) const;
-
-    void deleteHighlighting(KDevelop::DUContext* context) const;
+    /// This function is thread-safe
+    void highlightDUChain(ReferencedTopDUContext context);
 
     //color should be zero when undecided
     KTextEditor::Attribute::Ptr attributeForType(Types type, Contexts context, const QColor &color) const;
     KTextEditor::Attribute::Ptr attributeForDepth(int depth) const;
 
+    /// This function is thread-safe
+    /// Returns whether a highlighting is already given for the given url
+    bool hasHighlighting(IndexedString url) const;
+
+  private:
+    //Returns whether the given attribute was set by the code highlighting, and not by something else
+    //Always returns true when the attribute is zero
+    bool isCodeHighlight(KTextEditor::Attribute::Ptr attr) const;
+
   protected:
     //Can be overridden to create an own instance type
     virtual CodeHighlightingInstance* createInstance() const;
 
   private:
-    //Returns whether the given attribute was set by the code highlighting, and not by something else
-    //Always returns true when the attribute is zero
-    bool isCodeHighlight(KTextEditor::Attribute::Ptr attr) const;
+
+    /// Highlighting of one specific document
+    struct DocumentHighlighting
+    {
+      IndexedString m_document;
+      qint64 m_waitingRevision;
+      // The ranges are sorted by range start, so they can easily be matched
+      QVector<HighlightedRange> m_waiting;
+      QVector<KTextEditor::MovingRange*> m_highlightedRanges;
+    };
+
+    QMap<DocumentChangeTracker*, DocumentHighlighting*> m_highlights;
+
+    void clearHighlightingForDocument(KDevelop::IndexedString document);
 
     friend class CodeHighlightingInstance;
 
@@ -161,8 +201,15 @@ class KDEVPLATFORMLANGUAGE_EXPORT CodeHighlighting : public QObject, public KDev
     mutable QMutex m_dataMutex;
 
   private Q_SLOTS:
+    void applyHighlighting(void* highlighting);
+
+    void trackerDestroyed(QObject* object);
+
     /// when the colors change we must invalidate our local caches
     void adaptToColorChanges();
+
+    void aboutToInvalidateMovingInterfaceContent(KTextEditor::Document*);
+    void aboutToRemoveText(const KTextEditor::Range&);
 };
 
 }
