@@ -16,8 +16,8 @@
    Boston, MA 02110-1301, USA.
 */
 
-#ifndef DECLARATION_LIST_ITEM_QUICKOPEN
-#define DECLARATION_LIST_ITEM_QUICKOPEN
+#ifndef DECLARATION_LIST_QUICKOPEN_H
+#define DECLARATION_LIST_QUICKOPEN_H
 
 #include "duchainitemquickopen.h"
 
