@@ -122,7 +122,6 @@ public:
     Qt::ItemFlags flags;
     ProjectModel* model;
     KUrl m_url;
-    IndexedString m_baseName;
 };
 
 
@@ -197,8 +196,43 @@ void ProjectBaseItem::removeRow( int row )
 
 void ProjectBaseItem::removeRows(int row, int count)
 {
-    for( ; count > 0; count-- ) {
-        removeRow( row );
+    if (!count) {
+        return;
+    }
+
+    Q_D(ProjectBaseItem);
+    Q_ASSERT(row >= 0 && row + count <= d->children.size());
+
+    if( model() ) {
+        QMetaObject::invokeMethod( model(), "rowsAboutToBeRemoved", getConnectionTypeForSignalDelivery( model() ), Q_ARG(QModelIndex, index()), Q_ARG(int, row), Q_ARG(int, row + count - 1) );
+    }
+    QList<ProjectBaseItem*> toRemove;
+#if QT_VERSION >= 0x040700
+    toRemove.reserve(count);
+#endif
+    if (row == 0 && count == d->children.size()) {
+        // optimize shutdown for big projects
+        toRemove = d->children;
+        d->children.clear();
+    } else {
+        for (int i = row; i < count; ++i) {
+            toRemove << d->children.takeAt( row );
+        }
+        for(int i = row; i < d->children.size(); ++i) {
+            d->children.at(i)->d_func()->row--;
+            Q_ASSERT(child(i)->d_func()->row==i);
+        }
+    }
+
+    foreach(ProjectBaseItem* item, toRemove) {
+        item->d_func()->parent = 0;
+        item->d_func()->row = -1;
+        item->setModel( 0 );
+        delete item;
+    }
+
+    if( model() ) {
+        QMetaObject::invokeMethod( model(), "rowsRemoved", getConnectionTypeForSignalDelivery( model() ), Q_ARG(QModelIndex, index()), Q_ARG(int, row), Q_ARG(int, row + count - 1) );
     }
 }
 
@@ -357,14 +391,7 @@ KUrl ProjectBaseItem::url( ) const
 
 QString ProjectBaseItem::baseName() const
 {
-    Q_D(const ProjectBaseItem);
-    return d->m_baseName.str();
-}
-
-IndexedString ProjectBaseItem::indexedBaseName() const
-{
-    Q_D(const ProjectBaseItem);
-    return d->m_baseName;
+    return text();
 }
 
 void ProjectBaseItem::setUrl( const KUrl& url )
@@ -372,7 +399,6 @@ void ProjectBaseItem::setUrl( const KUrl& url )
     Q_D(ProjectBaseItem);
     d->m_url = url;
     const QString baseName = url.fileName();
-    d->m_baseName = IndexedString(baseName);
     setText( baseName );
 }
 
@@ -560,13 +586,13 @@ ProjectBaseItem::RenameStatus ProjectFolderItem::rename(const QString& newname)
 
 bool ProjectFolderItem::hasFileOrFolder(const QString& name) const
 {
-    const IndexedString indexedName(name);
-    for ( int i = 0; i < rowCount(); ++i )
+    foreach ( ProjectBaseItem* item, children() )
     {
-        ProjectBaseItem* item = child( i );
-        const IndexedString& other = item->indexedBaseName();
-        if ( other == indexedName )
+        if ( (item->type() == Folder || item->type() == File || item->type() == BuildFolder)
+             && name == item->baseName() )
+        {
             return true;
+        }
     }
     return false;
 }
@@ -841,11 +867,9 @@ ProjectVisitor::ProjectVisitor()
 
 QModelIndex ProjectModel::index( int row, int column, const QModelIndex& parent ) const
 {
-    if( hasIndex( row, column, parent ) ) {
-        ProjectBaseItem* parentItem = d->itemFromIndex( parent );
-        if( parentItem && row >= 0 && row < parentItem->rowCount() && column == 0 ) {
-            return createIndex( row, column, parentItem );
-        }
+    ProjectBaseItem* parentItem = d->itemFromIndex( parent );
+    if( parentItem && row >= 0 && row < parentItem->rowCount() && column == 0 ) {
+        return createIndex( row, column, parentItem );
     }
     return QModelIndex();
 }
