@@ -78,10 +78,6 @@ public:
     /**Adds the @p view to the list of views at the given area index.*/
     void addView(View *view, AreaIndex *index);
 
-    /**Adds the @p view to the list of views at the given area index.
-       Does not emit viewAdded. @warning This may lead to an inconsistent state in main-windows, unless their contents is reconstructed*/
-    void addViewSilently(View *view, AreaIndex *index);
-    
     /**Adds the @p view to the list of views at the current area index.*/
     void addView(View *view, View *after = 0);
     /**Adds the @p view to the area splitting the @p viewToSplit using
