@@ -24,16 +24,18 @@
 
 #include "vcsexport.h"
 #include <QtCore/QList>
-#include <kurl.h>
-#include <QHash>
+#include <QtCore/QHash>
+
+#include <KDE/KUrl>
+
 
 namespace KDevelop
 {
 /**
- * Denotes a local or repository location for a Vcs system
+ * Denotes a local or repository location for a Vcs system.
  *
- * For the RepositoryLocation type most of the information
- * are vcs specific
+ * For the RepositoryLocation type, most of the information
+ * is vcs-specific.
  */
 class KDEVPLATFORMVCS_EXPORT VcsLocation
 {
@@ -57,27 +59,27 @@ public:
     KUrl localUrl() const;
 
     /**
-     * Returns a string for the repository, usually this identifies the server
+     * Returns a string for the repository, usually this identifies the server.
      * @returns a vcs-implementation-specific string identifying the server
      */
     QString repositoryServer() const;
     /**
-     * Returns the module or module path inside the server
+     * Returns the module or module path inside the server.
      * @returns a vcs-implementation-specific string identifying the module
      */
     QString repositoryModule() const;
     /**
-     * identifies the tag which this location belongs to
+     * Identifies the tag which this location belongs to.
      * @returns a vcs-implementation-specific string identifying the tag
      */
     QString repositoryTag() const;
     /**
-     * identifies the branch to which this location belongs to
+     * Identifies the branch to which this location belongs to.
      * @returns a vcs-implementation-specific string identifying the branch
      */
     QString repositoryBranch() const;
     /**
-     * This can define a path relative to the module, this is used
+     * This can define a path relative to the module. This is used
      * when identifying a subdirectory or file inside a repository location
      * @returns a path relative to module
      */
@@ -115,8 +117,8 @@ public:
     void setRepositoryPath( const QString& );
 
     /**
-     * Allows to add vcs-specific data to this location
-     * automatically sets the type to RepositoryLocation
+     * Allows to add vcs-specific data to this location.
+     * Automatically sets the type to RepositoryLocation
      * @param data the vcs-specific data
      */
     void setUserData( const QVariant& );
