@@ -70,6 +70,9 @@
 #include <qlabel.h>
 #include <QMenu>
 #include "widgets/vcsdiffpatchsources.h"
+#include <interfaces/isession.h>
+#include "vcsevent.h"
+#include <KCompositeJob>
 
 namespace KDevelop
 {
@@ -87,6 +90,7 @@ struct VcsPluginHelper::VcsPluginHelperPrivate {
     KAction * diffToHeadAction;
     KAction * diffToBaseAction;
     KAction * revertAction;
+    KAction * diffForRevAction;
     
     void createActions(QObject * parent) {
         commitAction = new KAction(KIcon("svn-commit"), i18n("Commit..."), parent);
@@ -97,6 +101,7 @@ struct VcsPluginHelper::VcsPluginHelperPrivate {
         revertAction = new KAction(KIcon("archive-remove"), i18n("Revert"), parent);
         historyAction = new KAction(KIcon("view-history"), i18n("History..."), parent);
         annotationAction = new KAction(KIcon("user-properties"), i18n("Annotation..."), parent);
+        diffForRevAction = new KAction(KIcon("vcs_diff"), i18n("Show Diff..."), parent);
         
         connect(commitAction, SIGNAL(triggered()), parent, SLOT(commit()));
         connect(addAction, SIGNAL(triggered()), parent, SLOT(add()));
@@ -106,6 +111,7 @@ struct VcsPluginHelper::VcsPluginHelperPrivate {
         connect(revertAction, SIGNAL(triggered()), parent, SLOT(revert()));
         connect(historyAction, SIGNAL(triggered()), parent, SLOT(history()));
         connect(annotationAction, SIGNAL(triggered()), parent, SLOT(annotation()));
+        connect(diffForRevAction, SIGNAL(triggered()), parent, SLOT(diffForRev()));
     }
     
     bool allLocalFiles(const KUrl::List& urls)
@@ -254,7 +260,7 @@ void VcsPluginHelper::diffJobFinished(KJob* job)
         KDevelop::VcsDiff d = vcsjob->fetchResults().value<KDevelop::VcsDiff>();
         if(d.isEmpty())
             KMessageBox::error(ICore::self()->uiController()->activeMainWindow(),
-                                i18n("Cannot show the differences because there were none."),
+                                i18n("There are no differences."),
                                 i18n("VCS support"));
         else {
             VCSDiffPatchSource* patch=new VCSDiffPatchSource(d);
@@ -277,6 +283,22 @@ void VcsPluginHelper::diffToBase()
     ICore::self()->runController()->registerJob(job);
 }
 
+void VcsPluginHelper::diffForRev()
+{
+    QAction* action = qobject_cast<QAction*>( sender() );
+    Q_ASSERT(action);
+    Q_ASSERT(action->data().canConvert<VcsRevision>());
+    VcsRevision rev = action->data().value<VcsRevision>();
+
+    SINGLEURL_SETUP_VARS
+    ICore::self()->documentController()->saveAllDocuments();
+    VcsRevision prev = KDevelop::VcsRevision::createSpecialRevision(KDevelop::VcsRevision::Previous);
+    KDevelop::VcsJob* job = iface->diff(url, prev, rev );
+
+    connect(job, SIGNAL(finished(KJob*)), this, SLOT(diffJobFinished(KJob*)));
+    d->plugin->core()->runController()->registerJob(job);
+}
+
 void VcsPluginHelper::history()
 {
     SINGLEURL_SETUP_VARS
@@ -318,6 +340,9 @@ void VcsPluginHelper::annotation()
             KDevelop::VcsAnnotationModel* model = new KDevelop::VcsAnnotationModel(job, url, doc->textDocument());
             annotateiface->setAnnotationModel(model);
             viewiface->setAnnotationBorderVisible(true);
+            connect(doc->textDocument()->activeView(),
+                    SIGNAL(annotationContextMenuAboutToShow( KTextEditor::View*, QMenu*, int )),
+                    this, SLOT(annotationContextMenuAboutToShow( KTextEditor::View*, QMenu*, int )));
         } else {
             KMessageBox::error(0, i18n("Cannot display annotations, missing interface KTextEditor::AnnotationInterface for the editor."));
             delete job;
@@ -328,6 +353,19 @@ void VcsPluginHelper::annotation()
     }
 }
 
+void VcsPluginHelper::annotationContextMenuAboutToShow( KTextEditor::View* view, QMenu* menu, int line )
+{
+    KTextEditor::AnnotationInterface* annotateiface =
+        qobject_cast<KTextEditor::AnnotationInterface*>(view->document());
+
+    VcsAnnotationModel* model = qobject_cast<VcsAnnotationModel*>( annotateiface->annotationModel() );
+    Q_ASSERT(model);
+
+    VcsRevision rev = model->revisionForLine(line);
+    d->diffForRevAction->setData(QVariant::fromValue(rev));
+    menu->addAction(d->diffForRevAction);
+}
+
 void VcsPluginHelper::update()
 {
     EXECUTE_VCS_METHOD(update);
@@ -370,21 +408,20 @@ void VcsPluginHelper::commit()
     if(correctDiff)
         diff = diffJob->fetchResults().value<VcsDiff>();
     
-    if(!correctDiff) {
+    if(!correctDiff)
         KMessageBox::error(0, i18n("Could not create a patch for the current version."));
-    } else if(diff.isEmpty()) {
-        KMessageBox::information(0, i18n("Could not find any modifications to commit."));
+
+    // We start the commit UI no matter whether there is real differences, as it can also be used to commit untracked files
+    VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(diff, changes, d->vcs, retrieveOldCommitMessages());
+    
+    bool ret = showVcsDiff(patchSource);
+    
+    Q_ASSERT(ret && "Make sure PatchReview plugin is installed correctly");
+    if(ret) {
+        connect(patchSource, SIGNAL(reviewFinished(QString,QList<KUrl>)), this, SLOT(executeCommit(QString,QList<KUrl>)));
+        connect(patchSource, SIGNAL(reviewCancelled(QString)), this, SLOT(commitReviewCancelled(QString)));
     } else {
-        VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(diff, changes, d->vcs);
-        
-        bool ret = showVcsDiff(patchSource);
-        
-        Q_ASSERT(ret && "Make sure PatchReview plugin is installed correctly");
-        if(ret) {
-            connect(patchSource, SIGNAL(reviewFinished(QString,QList<KUrl>)), this, SLOT(executeCommit(QString,QList<KUrl>)));
-        } else {
-            delete patchSource;
-        }
+        delete patchSource;
     }
 }
 
@@ -392,6 +429,12 @@ void VcsPluginHelper::executeCommit(const QString& message, const QList<KUrl>& u
 {
     VcsJob* job=d->vcs->commit(message, urls, KDevelop::IBasicVersionControl::NonRecursive);
     d->plugin->core()->runController()->registerJob(job);
+    addOldCommitMessage(message);
+}
+
+void VcsPluginHelper::commitReviewCancelled(QString message)
+{
+    addOldCommitMessage(message);
 }
 
 void VcsPluginHelper::cancelCommit(KDevelop::VcsCommitDialog* dlg)
@@ -399,6 +442,35 @@ void VcsPluginHelper::cancelCommit(KDevelop::VcsCommitDialog* dlg)
     dlg->deleteLater();
 }
 
+QStringList retrieveOldCommitMessages()
+{
+    KConfigGroup vcsGroup(ICore::self()->activeSession()->config(), "VCS");
+    return vcsGroup.readEntry("OldCommitMessages", QStringList());
 }
 
+namespace {
+    int maxMessages = 10;
+}
+
+void addOldCommitMessage(QString message)
+{
+    if(ICore::self()->shuttingDown())
+        return;
+    
+    QStringList oldMessages = retrieveOldCommitMessages();
+    
+    if(oldMessages.contains(message))
+        oldMessages.removeAll(message);
+    
+    oldMessages.push_front(message);
+    while(oldMessages.size() > maxMessages)
+        oldMessages.pop_back();
+    
+    KConfigGroup vcsGroup(ICore::self()->activeSession()->config(), "VCS");
+    vcsGroup.writeEntry("OldCommitMessages", oldMessages);
+}
+
+}
+
+
 #include "vcspluginhelper.moc"
